#!/usr/bin/perl
###############################################################################
#
#	Kovertiert TAP-Programme des Z9001 (binres Format) in WAV-Dateien  
#	Nutzung der Ideen von H. Haftmann (KCLOAD+KCEMU)
#
#	22.02.2016 	by Volker Pohlers created
#
###############################################################################

use strict;
use Fcntl ":seek";

# $ARGV[0] = 'd:/hobby/z9001 devdoku/perl/wav2tap/test_.tap';

die <<"help" unless @ARGV;
$0 tap-file
konvertiert TAP-Dateien in WAV-Dateien
Wave-Datei: 8 Bit, 22 kHz, Mono
help

our $block;
our $filesize;

########################################
# TAP-Datei ffnen
########################################

open IN, $ARGV[0] or die;
binmode IN;

(our $WAV = $ARGV[0]) =~ s/\.tap/\.wav/i;
die "Fehler: keine TAP-Datei!" if $WAV =~ /$ARGV[0]/i;

# Header lesen
read IN, $block, 16; 
if ($block ne 'KC-TAPE by AF. ') {
	close IN;
	print 'Kein TAP-Format!';
	exit;
}

########################################
# WAV-Datei erstellen
########################################

print "Schreibe $WAV...";
open WAV, ">$WAV" or die;
binmode WAV;
# WAV-Header
print WAV "RIFF\x00\x00\x00\x00WAVEfmt \x10\x00\x00\x00\x01\x00\x01\x00\x22\x56\x00\x00\x22\x56\x00\x00\x01\x00\x08\x00data\x00\x00\x00\x00";


#Ausgabe 
our $Vorton = 0x1770;	# mit langem Vorton beginnen
our $Amp = 0x20;	# Momentanwert (also Elongation)

# Blcke schreiben
until (eof IN) {
	# Blocknummer + Block
	read IN, $block, 129; 
	my $blknr = substr($block,0,1);
	
	printf "%.2lX ", ord $blknr;
	
	# Ausgabe als WAV
	WriteKCBlock($Vorton, $block);
	# nach einem FF-Block langen Vorton (z.B. OS-SAVE)
	if (ord $blknr == 255) { $Vorton = 0x14B0 } else { $Vorton = 0x00A0 }
}

$filesize = tell WAV;
close WAV;
close IN;


########################################
# WAV-"Korrektur"
########################################

open WAV, "+<$WAV" or die;
binmode WAV;

seek WAV, 4, SEEK_SET;
print WAV pack('L', $filesize-8);

seek WAV, 40, SEEK_SET;
print WAV pack('L', $filesize-44);

close WAV;

# Fertig!
print "ok\n";


########################################
# Save-Routinen Z9001
########################################

# Blockaufbau Z9001: Vorton 1 Byte Blocknummer, 128 Byte, 1 Byte Prfsumme 

sub WriteKCBlock {
	($Vorton, $block) = @_;
	my $sum = 0;
  	for (my $i = 1; $i <= $Vorton; $i++) {
  		WriteBit1() # Vorton
  	}
 	WriteSync();	#Trennzeichen
 	WriteKCByte(ord substr($block,0,1));	# Blocknummer
  	for (my $i = 1; $i <= 128; $i++) {
  		my $byte = ord substr($block,$i,1);
 		WriteKCByte($byte);		# 128 Bytes
 		$sum += $byte;
 	}
	WriteKCByte($sum);			# Prfsumme
}

# Byteaufbau Z9001: 0-Bit 2500 Hz, 1-Bit 1250 Hz, Trennzeichen 625 Hz
# Die Frequenzangaben stammen aus dem Handbuch und entsprechen nicht der
# Realitt! 

sub WriteKCByte {
	my $byte = shift;
	# printf ("%.2X ", $byte);

	for (my $i=0; $i<=7; $i++) {
		if ($byte & 1) { WriteBit1() } else { WriteBit0() };
		$byte >>= 1;
	}
	WriteSync();
}		

# Ganze Schwingung der gewnschten Lnge (in 22-kHz-Samples) schreiben
# nach KCLOAD v. H. Haftmann

sub WriteSync { WriteSwing2(39) }	# Sync
sub WriteBit0 { WriteSwing2(11) }	# 0-Bit
sub WriteBit1 { WriteSwing2(21) }	# 1-Bit

sub WriteSwing2 {
	my $Len = shift;
	my $Len1 = int $Len/2;
	WriteSwing1($Len1);
	WriteSwing1($Len-$Len1);
}

# Schreibt eine Halbschwingung der gewnschten Lnge mit abgerundeter
# Vorderflanke (nur bei Lngen >=4) - ansonsten war die Aufzeichnung futsch
# sowie keine KC-Direktankopplung (ohne Magnetband) mglich.}

sub WriteSwing1 {
	my $Len = shift;
	#print "$Len*$Amp ";
	my $AmpOrig;
	if ($Len >= 4) {
	    $AmpOrig = -$Amp;
	    $Amp = 0; WriteB();
	    $Amp = int $AmpOrig / 2; WriteB();
	    $Amp = int ($AmpOrig * 3) / 4; WriteB();
	    $Amp = int ($AmpOrig * 7) / 8; WriteB();
	    $Amp = $AmpOrig;
	    $Len -= 4;
	} else {
	    $Amp = -$Amp;
	}
	while ($Len != 0) {
	    WriteB();	# Rest gerade (Gleichspannung)
	    $Len--;
	}
}

# Schreibt ein Sample "Amp" (globale Variable) auf die Wave-Ausgabe
sub WriteB {
	print WAV chr($Amp+0x80);
}

__END__
