#!/usr/bin/perl
###############################################################################
#
#	Kovertiert WAV-Dateien fr den Z9001 ins binre TAP-Format 
#
#	12.02.2016	by Volker Pohlers created
#
###############################################################################

use strict;
use Fcntl ":seek";

#$ARGV[0] = 'd:/hobby/z9001 prettyc//Sieb_.wav';

die <<"help" unless @ARGV;
konvertiert WAV-Dateien mit Z9001-Programm in TAP-Datei
Wave-Datei: 8 Bit, 22 kHz, Mono

- Es wird nicht kontrolliert, ob die WAV-Datei den Vorgaben entspricht
- ein (berflssiger) Kopfblock 0 wird bei KC87-Basic-Dateien nicht entfernt
- Der Dateiname wird fr tap genutzt, ggf. tap_rename.pl starten

help

our $bufsize = 8*2**10;	# 8k
our $pos = $bufsize;
our $buf;
our $offset;
our $abort;

open IN, $ARGV[0];
binmode IN;
seek IN, 54, SEEK_CUR;	# Wave-Header berspringen


#########################################
# WAV lesen
#########################################

# ein Byte lesen
sub ReadB {
	my $b;
	
	if ($pos == $bufsize) {
		# print "wav$bufsize ";
		$bufsize = read(IN, $buf, $bufsize);
		$abort = 1 if $bufsize == 0;
		$pos = 0;
	}
	$b = ord(substr($buf, $pos++, 1));
	return $b;
}


# ein Flankenwechsel abwarten (1/2 Periode)
sub ReadSwing1 {
	my $w;
	
	$w = 1;
	if (ReadB() < $offset)	{ do { $w++ } until (ReadB() > $offset) or $abort }
	else			{ do { $w++ } until (ReadB() < $offset) or $abort }

	return $w;
}

# eine volle Periode
sub ReadSwing2 {
	my $ww;
	
	$ww = ReadSwing1() + ReadSwing1();

	return $ww;
}

#########################################
# Formatabhngige Lese-Routinen 
#########################################


#### Z9001 mit Blocknummer
# Wave-Datei: 8bit, 22khz, mono

sub KCByteIn {
	my $i, my $w, my $b;
	$b=0;

	for( $i=0;$i<8;$i++) {
	    $w = ReadSwing2();
	    $b = $b >> 1;
	    if ($w>16) { $b |= 0x80 }
	}
	ReadSwing2();

	return $b;
}

sub ReadBlock {
	(my $BlkNr, my $Buffer) = @_;
    	my $I,my $W,my $B,my $Sum;

	#print "vorton\n";
	
  	# Schritt 1: Vorton erkennen und aufsynchronisieren
  	for ($I=1;$I<=22;$I++) {
	     	$W = ReadSwing2();
     		if (($W<18) or ($W>30)) {
      			$I = 0;
      			next;
     		}
    	}
  
#print "\n", tell (IN)+$pos, "\n";
	#print "Trennzeichen\n";
	
	# schritt 2: 1. Trennzeichen holen
  	for ($I=1;$I<=2;$I++) {
	     	$W = ReadSwing1();
     		if ($W<15) {
      			$I = 0;
      			next;
     		}
	}

#print "\n", tell (IN)+$pos, "\n";
	#print "Bytes\n";

	# Schritt 3: Bytes lesen
    	$$BlkNr = KCByteIn();
    	$$Buffer[0] = $$BlkNr;
    	$Sum = 0;
    	for ($I=1;$I<=128;$I++) {
		$B = KCByteIn();
     		$Sum += $B;
     		$$Buffer[$I] = $B;
    	}
    	
    	$Sum &=0xFF;	# auf Byte beschrnken
    	#print "$BlkNr\n";
    	#print "---",$Sum,"---";
    	return ( $Sum==KCByteIn() );
}

# Header anzeigen (Block 0)
sub kcc_header{
	my $Buffer = pack("C*",@_);
	my $name, my $typ,my $aadr,my $eadr,my $sadr;
	my $blknr;
	
	#KC-Header
	($blknr, $name, $typ, , , , , , , $aadr, $eadr, $sadr) = unpack("
		a		   # blknr
		a8		   # dateiname: array[0..7] of char;
		a3		   # dateityp: array[0..2] of char;
		xx		   # e1, e2: byte;
		xxxx		   # psum, arb, blnr, lblnr: byte;
		SSS		   # aadr, eadr, sadr: word;
		x		   # sby: byte;
		", $Buffer
	);
	
	$name =~ s/\x00//g;
	$typ =~ s/\x00//g;
	
	printf ("name %s, typ %s, aadr %.4lX, eadr %.4lX, sadr %.4lX\n",
	$name,$typ,$aadr,$eadr,$sadr);
	
	return $name.'_'.$typ;
}

#########################################
# Main
#########################################


our $BlkNr;
our $I, our $W;
our @buf;
our @tapbuf;
our $name;

# Mittelwert bestimmen (Gleichspannungsanteil)
# JKCEmu produziert Werte 0x80 und 0x5C

{
	my $min = 0xff;
	my $max = 0;
	for $I (1..1000) {
		my $b = ReadB();
		$min = $b if $min > $b;
		$max = $b if $max < $b;
	}
	$offset = ($max+$min)/2;
	print "DC-Offset $offset\n";
	# Puffer rcksetzen
	$pos = 0;
}


# Vorton
print "Vorton\n";
for ($I = 300;$I>=0;$I--) {
	$W = ReadSwing2();
	if (($W<18) or ($W>30)) { $I=200 };
	if ($abort) { print "Dateieende!\n"; exit }
}

# Blcke
print "Bloecke\n";
while ( !eof(IN) ) {
	if (ReadBlock(\$BlkNr,\@buf)) {
		printf "%.2X ",$BlkNr;
		push (@tapbuf, @buf);
		if ($BlkNr == 0) {
			$name = kcc_header(@buf);
		}
	} else { 
		print "\nERROR->",$BlkNr;
	}
}

close IN;

#########################################
# TAP-Datei schreiben
#########################################

open OUT, '>'. ($name || 'test') . '.tap';
binmode OUT;
#Header schreiben
syswrite OUT, 'KC-TAPE by AF. ';
syswrite OUT, pack ("C*", @tapbuf); 
close OUT;


__END__

